@@ -24,7 +24,7 @@
 #ifndef NM_ACTIVE_CONNECTION_H
 #define NM_ACTIVE_CONNECTION_H
 
-#include <glib/gtypes.h>
+#include <glib.h>
 #include <glib-object.h>
 #include "nm-object.h"
 #include <nm-connection.h>
