@@ -1053,7 +1053,7 @@ real_act_stage2_config (NMDevice *device, NMDeviceStateReason *reason)
 		ppp_name = serial_class->get_ppp_name (NM_SERIAL_DEVICE (device), req);
 
 	priv->ppp_manager = nm_ppp_manager_new (nm_device_get_iface (device));
-	if (nm_ppp_manager_start (priv->ppp_manager, req, ppp_name, &err)) {
+	if (nm_ppp_manager_start (priv->ppp_manager, req, ppp_name, 20, &err)) {
 		g_signal_connect (priv->ppp_manager, "state-changed",
 					   G_CALLBACK (ppp_state_changed),
 					   device);
