@@ -24,15 +24,17 @@
 #include <glib.h>
 #include <nm-connection.h>
 
-#define TYPE_ETHERNET "Ethernet"
-#define TYPE_WIRELESS "Wireless"
+#include "shvar.h"
 
 NMConnection *connection_from_file (const char *filename,
                                     const char *network_file,
                                     const char *test_type,
                                     gboolean *ignored,
                                     char **keyfile,
+                                    char **routefile,
                                     GError **error,
                                     gboolean *ignore_error);
 
+const char *reader_get_prefix (void);
+
 #endif  /* __READER_H__ */
